'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

//      
// reference: https://redux.js.org/docs/recipes/ReducingBoilerplate.html#generating-action-creators






function makeActionCreator(type) {for (var _len = arguments.length, argNames = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {argNames[_key - 1] = arguments[_key];}
  return function actionCreator() {for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {args[_key2] = arguments[_key2];}
    var action = { type: type };
    argNames.forEach(function (arg, index) {
      action[argNames[index]] = args[index];
    });
    return action;
  };
}

function isNullOrUndefined(value) {
  return value === null || value === undefined;
}

function defaultActionIdGetter(action) {
  if (isNullOrUndefined(action.id)) {
    throw Error("Action should have an 'id' key");
  }

  return action.id;
}

function defaultPayloadIdGetter(datum) {
  if (isNullOrUndefined(datum.id)) {
    throw Error("Payload objects should have an 'id' key");
  }

  return datum.id;
}

function defaultActionPayloadGetter(action) {
  if (isNullOrUndefined(action.data)) {
    throw Error("Action does not contain payload with the key 'data'");
  }

  return action.data;
}

/**
   *
   * @typedef Model
   * @property {String} INSERT_MANY
   * @property {String} INSERT
   * @property {String} UPDATE
   * @property {String} DELETE
   * @property {function(data: Array.<T>)} insertMany
   * @property {function(id: String, data: T)} insert
   * @property {function(id: String, data: T)} update
   * @property {function(id: String)} delete
   * @template T
   */












/**
       * @param namespace {String}
       * @return {Model}
       */
function makeActionsAndActionCreatorsForThings(namespace) {
  var INSERT_MANY = "[".concat(namespace, "] INSERT_MANY");
  var INSERT = "[".concat(namespace, "] INSERT");
  var UPDATE = "[".concat(namespace, "] UPDATE");
  var DELETE = "[".concat(namespace, "] DELETE");

  var model = {
    INSERT_MANY: INSERT_MANY,
    INSERT: INSERT,
    UPDATE: UPDATE,
    DELETE: DELETE,
    insertMany: makeActionCreator < Array < Model >> ('data'),
    insert: makeActionCreator < Model > ('data'),
    update: makeActionCreator < Model > ('data'),
    "delete": makeActionCreator < Model > ('id') };


  return model;
}

/**
   * @param {Model} model
   * @param [opts]
   * @param [opts.getActionId=defaultActionIdGetter] {Function}
   * @param [opts.getPayloadId=defaultPayloadIdGetter] {Function}
   * @param [opts.getActionPayload=defaultActionPayloadGetter] {Function}
   * @returns {function(state, action)} reducer
   */
function makeNormalizedReducer(client) {var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};var _opts$getActionId =




  opts.getActionId,getActionId = _opts$getActionId === void 0 ? defaultActionIdGetter : _opts$getActionId,_opts$getPayloadId = opts.getPayloadId,getPayloadId = _opts$getPayloadId === void 0 ? defaultPayloadIdGetter : _opts$getPayloadId,_opts$getActionPayloa = opts.getActionPayload,getActionPayload = _opts$getActionPayloa === void 0 ? defaultActionPayloadGetter : _opts$getActionPayloa;

  return function reducer() {var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};var action = arguments.length > 1 ? arguments[1] : undefined;
    var newState,
    payload,
    id;

    switch (action.type) {
      case client.INSERT_MANY:
        newState = Object.assign({}, state);
        getActionPayload(action).forEach(function (value) {
          newState[getPayloadId(value)] = value;
        });
        return newState;
      case client.INSERT:
        payload = getActionPayload(action);
        id = getActionId(action) || getPayloadId(payload);
        return _objectSpread({}, state, _defineProperty({},
        id, getActionPayload(action)));

      case client.UPDATE:
        payload = getActionPayload(action);
        id = getActionId(action) || getPayloadId(payload);
        return _objectSpread({}, state, _defineProperty({},
        id, _objectSpread({},
        state[id],
        payload)));


      case client.DELETE:
        newState = _objectSpread({}, state);
        delete newState[getActionId(action)];
        return newState;
      default:
        return state;}

  };
}

exports.makeActionsAndActionCreatorsForThings = makeActionsAndActionCreatorsForThings;
exports.makeNormalizedReducer = makeNormalizedReducer;
